import { Component, ElementRef, ViewChild, ViewEncapsulation } from '@angular/core';
import * as common from "mindfusion-common";
import * as MindFusion from "diagram-library";
import Diagram = MindFusion.Diagramming.Diagram;
import CompositeCommand = MindFusion.Diagramming.CompositeCommand;
import ChangeItemCommand = MindFusion.Diagramming.ChangeItemCommand;
import AddItemCommand = MindFusion.Diagramming.AddItemCommand;
import RemoveItemCommand = MindFusion.Diagramming.RemoveItemCommand;
import Rect = common.Drawing.Rect;
import Event = MindFusion.Diagramming.Events;
import GraphicsUnit = common.Drawing.GraphicsUnit;
import CompositeNode = MindFusion.Diagramming.CompositeNode;
import Image = MindFusion.Drawing.Image;
import ShapeNode = MindFusion.Diagramming.ShapeNode;
import Font = MindFusion.Drawing.Font;
import SimplePanel = MindFusion.Diagramming.SimplePanel;
import Factory = MindFusion.Diagramming.Factory;
import Shape = MindFusion.Diagramming.Shape;
import DiagramLink = MindFusion.Diagramming.DiagramLink;
import GridRouter = common.Diagramming.GridRouter;
import Alignment = MindFusion.Diagramming.Alignment;
import Thickness = common.Drawing.Thickness;
import MIND = MindFusion.Diagramming;
import Behavior = MindFusion.Diagramming.Behavior;
import AnchorPoint = MindFusion.Diagramming.AnchorPoint;
import AnchorPattern = MindFusion.Diagramming.AnchorPattern;
import MarkStyle = MindFusion.Diagramming.MarkStyle;
import Point = common.Drawing.Point;
import LinkEventArgs = MindFusion.Diagramming.LinkEventArgs;
import { DiagramExportService } from './services/diagram-export.service';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.css'],
  encapsulation: ViewEncapsulation.None
})
export class AppComponent {

  @ViewChild('canvas', { static: true }) public canvas?: ElementRef;

  public diagram!: Diagram;

  public DiagramNode: any;

  public diagramElements!: Array<DiagramElement>;

  public backColorElement!: DiagramElement;
  public backColorNode!: CompositeNode;

  public isBackgroundGridShown: boolean = true;

  public reduceSize: number = 2;

  public isButtonVisible = true;

  private handleLinkCreated: any;
  private handleLinkModified: any;
  private handleNodeModified: any;

  constructor(private diagramExportService: DiagramExportService) {
    this.diagramExportService.isBackgroundGridShown = this.isBackgroundGridShown;
  }

  ngAfterViewInit(): void {
    this.diagram = Diagram.create(this.canvas?.nativeElement);

    this.diagram.setBackBrush("Transparent");
    this.diagram.setAutoResize(MindFusion.Diagramming.AutoResize.RightAndDown);
    this.diagram.setRouteLinks(true);
    this.diagram.setRoundedLinks(true);
    this.diagram.setMeasureUnit(GraphicsUnit.WpfPoint);
    this.diagram.setLinkHeadShapeSize(10);
    this.diagram.setAutoScroll(true);
    this.diagram.setVirtualScroll(true);
    this.diagram.setBehavior(MindFusion.Diagramming.Behavior.DrawLinks);
    this.diagram.setModificationStart(MindFusion.Diagramming.ModificationStart.AutoHandles);
    this.diagram.setEnabled(true);
    this.diagram.setAllowInplaceEdit(true);
    var selectionHandleSize = 6;
    this.diagram.setAdjustmentHandlesSize(selectionHandleSize);
    this.setLinkGridRouter();

    this.diagram.setShowGrid(false);
    this.diagram.setGridStyle(common.Diagramming.GridStyle.Lines);
    this.diagram.setGridSizeX(this.diagram.getGridSizeX() + 5);
    this.diagram.setGridSizeY(this.diagram.getGridSizeY() + 5);

    this.diagram.setLinkBaseShapeSize(10);

    this.diagram.setAllowSelfLoops(false);
    this.diagram.setLinkCrossings(common.Diagramming.LinkCrossings.Arcs);
    this.diagram.setCrossingRadius(6);
    this.diagram.setRoundedLinks(true);
    this.diagram.setRoundedLinksRadius(6);

    var ths = this;
    this.handleLinkCreated = function (sender: any, args: LinkEventArgs) {
      var link = args.getLink() as any;
      ths.consoleLinkPoints(link, "On Create: ");
    }

    this.handleLinkModified = function (sender: any, args: LinkEventArgs) {

      var link = args.getLink() as any;

      if (link) {
        ths.consoleLinkPoints(link, "On Modify: ");
      }
    }

    this.handleNodeModified = function (sender: any, args: any) {

      var node = args.getNode();

      var link = node.incomingLinks[0];
      if (link) {
        ths.consoleLinkPoints(link, "On Modify: ");
      }
    }
    this.diagram.addEventListener(Event.linkCreated, this.handleLinkCreated);

    this.diagram.addEventListener(Event.linkModified, this.handleLinkModified);

    this.diagram.addEventListener(Event.nodeModified, this.handleNodeModified);

    this.loadNodeTemplate();

    this.diagramExportService.diagram = this.diagram;
  }

  private consoleLinkPoints(link: any, message: string) {

    console.log(message)

    var points = link.getControlPoints();

    for (var i = 0; i < points.length; i++) {
      var point = points[i];
      console.log("Point " + i + ": " + point.x + ", " + point.y);
    }
  }

  public exportDiagramClick(args: any): void {

    var type = "png";
    var fileName = "canvas-normal";

    this.diagramExportService.exportDiagram(false, fileName, type);
  }

  public exportDiagramHRClick(args: any): void {

    var fileName = "canvas-normal-hr";

    this.diagramExportService.exportDiagram(true, fileName);
  }

  public exportDiagramFitClick(args: any): void {

    var fileName = "canvas-fit";

    this.diagramExportService.exportDiagramFit(this.reduceSize, false, fileName);
  }

  public exportDiagramHRFitClick(args: any): void {

    var fileName = "canvas-fit-hr";

    this.diagramExportService.exportDiagramFit(this.reduceSize, true, fileName);
  }

  //public onTextChange(event: any) {
  //  this.reduceSize = Number.parseInt((event.target as HTMLInputElement).value);
  //}

  public setLinkGridRouter() {
    var gridRouter = new GridRouter(this.diagram);
    gridRouter.setNodeVicinityCost(22);
    gridRouter.setNodeVicinitySize(22);
    this.diagram?.setLinkRouter(gridRouter);
  }

  public loadNodeTemplate(): void {

    var url = encodeURI("./assets/templates/diagram-node.template.txt");
    var xhr = new XMLHttpRequest();
    if (xhr.overrideMimeType)
      xhr.overrideMimeType("text/plain");
    xhr.open("GET", url);
    xhr.onload = (e) => {
      if (xhr.status === 200) {
        var content = xhr.responseText;
        var template = JSON.parse(content);
        this.DiagramNode = CompositeNode.classFromTemplate("DiagramNode", template);

        (MindFusion.Diagramming.Shape.shapes as any).Decision.params.outline += " Z";

        //this.diagram.setUndoEnabled(true);

        this.drawDiagram2();

        //this.setUndoRedoText();

        var viewPort = this.diagram.getViewport();
        var width = viewPort.width;
        var height = viewPort.height;

        //console.log('width:' + width);
        //console.log('height:' + height);

        var size1 = this.diagram.getScrollZoneSize();

        var size2 = this.diagram.getScrollZoneSize();
      }
      else
        console.log('Request failed status: ' + xhr.status);
    };
    xhr.onerror = function (e) {
      console.log('Request failed error: ' + xhr.status + ';' + e.toString());
    };
    xhr.send();
  }

  public drawDiagram(): void {

    this.diagramElements = new Array<DiagramElement>();

    this.clearDiagram();

    let factory = <Factory>this.diagram.getFactory();

    var diagramElement01 = new DiagramElement();
    diagramElement01.imageSource = "./assets/images/decisions/Decision-L-31.png";
    diagramElement01.name = "Decision-Lab-1";
    //diagramElement01.noOfElements = 12;
    diagramElement01.x = 10;
    diagramElement01.y = 120;
    diagramElement01.width = 48;
    diagramElement01.height = 48;
    diagramElement01.labelX = diagramElement01.x - 20;
    diagramElement01.labelY = diagramElement01.y + diagramElement01.height + 20;
    diagramElement01.labelWidth = 100;
    diagramElement01.labelHeight = 20;
    //diagramElement01.borderColor = "Blue";
    diagramElement01.shape = ElementShape.Decision;

    var diagramElement02 = new DiagramElement();
    diagramElement02.imageSource = "./assets/images/decisions/Decision-L-32.png";
    diagramElement02.name = "Decision-Lab-2";
    //diagramElement02.noOfElements = 12;
    diagramElement02.x = 100;
    diagramElement02.y = 120;
    diagramElement02.width = 48;
    diagramElement02.height = 48;
    diagramElement02.labelX = diagramElement02.x - 20;
    diagramElement02.labelY = diagramElement02.y + diagramElement02.height + 20;
    diagramElement02.labelWidth = 100;
    diagramElement02.labelHeight = 20;
    //diagramElement02.borderColor = "Blue";
    diagramElement02.shape = ElementShape.Decision;

    var diagramElement11 = new DiagramElement();
    diagramElement11.imageSource = "./assets/images/Circle1.png";
    diagramElement11.name = "Circle-1";
    //diagramElement11.noOfElements = 53;
    diagramElement11.isCheckedOutByMe = true;
    diagramElement11.isHasKnowledge = false;
    diagramElement11.x = 10;
    diagramElement11.y = 320;
    diagramElement11.width = 32;
    diagramElement11.height = 32;
    diagramElement11.labelX = diagramElement11.x - 20;
    diagramElement11.labelY = diagramElement11.y + diagramElement11.height + 20;
    diagramElement11.labelWidth = 100;
    diagramElement11.labelHeight = 20;
    diagramElement11.isShadowElement = true;
    diagramElement11.shape = ElementShape.Circle

    var diagramElement12 = new DiagramElement();
    diagramElement12.imageSource = "./assets/images/Circle2.png";
    diagramElement12.name = "Circle-2";
    //diagramElement12.noOfElements = 53;
    diagramElement12.isLocked = true;
    diagramElement12.x = 400;
    diagramElement12.y = 100;
    diagramElement12.width = 32;
    diagramElement12.height = 32;
    diagramElement12.labelX = diagramElement12.x - 20;
    diagramElement12.labelY = diagramElement12.y + diagramElement12.height + 20;
    diagramElement12.labelWidth = 100;
    diagramElement12.labelHeight = 20;
    diagramElement12.shape = ElementShape.Circle

    var diagramElement13 = new DiagramElement();
    diagramElement13.imageSource = "./assets/images/Circle3.png";
    diagramElement13.name = "Circle-3";
    //diagramElement13.noOfElements = 53;
    diagramElement13.isLocked = true;
    diagramElement13.x = 80;
    diagramElement13.y = 320;
    diagramElement13.width = 32;
    diagramElement13.height = 32;
    diagramElement13.labelX = diagramElement13.x - 20;
    diagramElement13.labelY = diagramElement13.y + diagramElement13.height + 20;
    diagramElement13.labelWidth = 100;
    diagramElement13.labelHeight = 20;
    diagramElement13.isShadowElement = false;
    diagramElement13.shape = ElementShape.Circle

    var diagramElement14 = new DiagramElement();
    diagramElement14.imageSource = "./assets/images/Circle4.png";
    diagramElement14.name = "Circle-4";
    //diagramElement14.noOfElements = 53;
    diagramElement14.isLocked = true;
    diagramElement14.x = 600;
    diagramElement14.y = 100;
    diagramElement14.width = 32;
    diagramElement14.height = 32;
    diagramElement14.labelX = diagramElement14.x - 20;
    diagramElement14.labelY = diagramElement14.y + diagramElement14.height + 20;
    diagramElement14.labelWidth = 100;
    diagramElement14.labelHeight = 20;
    diagramElement14.shape = ElementShape.Circle

    var diagramElement15 = new DiagramElement();
    diagramElement15.imageSource = "./assets/images/Circle5.png";
    diagramElement15.name = "Circle-5 (with yellow border)";
    //diagramElement15.noOfElements = 53;
    diagramElement15.isLocked = true;
    diagramElement15.x = 150;
    diagramElement15.y = 320;
    diagramElement15.width = 32;
    diagramElement15.height = 32;
    diagramElement15.labelX = diagramElement15.x - 20;
    diagramElement15.labelY = diagramElement15.y + diagramElement15.height + 20;
    diagramElement15.labelWidth = 100;
    diagramElement15.labelHeight = 20;
    diagramElement15.borderColor = "yellow";
    diagramElement15.isShadowElement = true;
    diagramElement15.shape = ElementShape.Circle

    var diagramElement31 = new DiagramElement();
    diagramElement31.isTest = true;
    diagramElement31.imageSource = "./assets/images/decisions/Decision-H1.png";
    diagramElement31.name = "Decision-H1";
    //diagramElement31.noOfElements = 23;
    diagramElement31.isCheckedOutByMe = true;
    diagramElement31.isLocked = true;
    diagramElement31.x = 10;
    diagramElement31.y = 230;
    diagramElement31.width = 35;
    diagramElement31.height = 35;
    diagramElement31.labelX = diagramElement31.x - 20;
    diagramElement31.labelY = diagramElement31.y + diagramElement31.height + 20;
    diagramElement31.labelWidth = 100;
    diagramElement31.labelHeight = 20;
    //diagramElement31.borderColor = "Blue";
    diagramElement31.shape = ElementShape.Decision;

    var diagramElement32 = new DiagramElement();
    diagramElement32.isTest = true;
    diagramElement32.imageSource = "./assets/images/decisions/Decision-H2.png";
    diagramElement32.name = "Decision-H2";
    //diagramElement32.noOfElements = 12;
    diagramElement32.x = 80;
    diagramElement32.y = 230;
    diagramElement32.width = 32;
    diagramElement32.height = 32;
    diagramElement32.labelX = diagramElement32.x - 20;
    diagramElement32.labelY = diagramElement32.y + diagramElement32.height + 20;
    diagramElement32.labelWidth = 100;
    diagramElement32.labelHeight = 20;
    //diagramElement32.borderColor = "Blue";
    diagramElement32.shape = ElementShape.Decision;

    var diagramElement33 = new DiagramElement();
    diagramElement33.isTest = true;
    diagramElement33.imageSource = "./assets/images/decisions/Decision-H3.png";
    diagramElement33.name = "Decision-H3";
    //diagramElement33.noOfElements = 12;
    diagramElement33.x = 150;
    diagramElement33.y = 230;
    diagramElement33.width = 32;
    diagramElement33.height = 32;
    diagramElement33.labelX = diagramElement33.x - 20;
    diagramElement33.labelY = diagramElement33.y + diagramElement33.height + 20;
    diagramElement33.labelWidth = 100;
    diagramElement33.labelHeight = 20;
    //diagramElement33.borderColor = "Blue";
    diagramElement33.shape = ElementShape.Decision;

    var diagramElement61 = new DiagramElement();
    diagramElement61.isTest = true;
    diagramElement61.imageSource = "./assets/images/decisions/Decision-S1.png";
    diagramElement61.name = "Decision-S1";
    //diagramElement61.noOfElements = 12;
    diagramElement61.isCheckedOutByMe = true;
    diagramElement61.isHasHyperlinks = true;
    diagramElement61.isLocked = true;
    diagramElement61.isHasKnowledge = false;
    diagramElement61.x = 10;
    diagramElement61.y = 30;
    diagramElement61.width = 32;
    diagramElement61.height = 32;
    diagramElement61.labelX = diagramElement61.x - 20;
    diagramElement61.labelY = diagramElement61.y + diagramElement61.height + 20;
    diagramElement61.labelWidth = 100;
    diagramElement61.labelHeight = 20;
    //diagramElement61.borderColor = "Blue";
    diagramElement61.shape = ElementShape.Decision;

    var diagramElement62 = new DiagramElement();
    diagramElement62.isTest = true;
    diagramElement62.imageSource = "./assets/images/decisions/Decision-S2.png";
    diagramElement62.name = "Decision-S2";
    //diagramElement62.noOfElements = 12;
    diagramElement62.x = 80;
    diagramElement62.y = 30;
    diagramElement62.width = 32;
    diagramElement62.height = 32;
    diagramElement62.labelX = diagramElement62.x - 20;
    diagramElement62.labelY = diagramElement62.y + diagramElement62.height + 20;
    diagramElement62.labelWidth = 100;
    diagramElement62.labelHeight = 20;
    //diagramElement62.borderColor = "Blue";
    diagramElement62.shape = ElementShape.Decision;

    var diagramElement63 = new DiagramElement();
    diagramElement63.isTest = true;
    diagramElement63.imageSource = "./assets/images/decisions/Decision-S3.png";
    diagramElement63.name = "Decision-S3";
    //diagramElement63.noOfElements = 12;
    diagramElement63.x = 150;
    diagramElement63.y = 30;
    diagramElement63.width = 32;
    diagramElement63.height = 32;
    diagramElement63.labelX = diagramElement63.x - 20;
    diagramElement63.labelY = diagramElement63.y + diagramElement63.height + 20;
    diagramElement63.labelWidth = 100;
    diagramElement63.labelHeight = 20;
    //diagramElement63.borderColor = "Blue";
    diagramElement63.shape = ElementShape.Decision;

    var diagramElement41 = new DiagramElement();
    diagramElement41.imageSource = "./assets/images/Nodes_6.png";
    diagramElement41.name = "Node-6";
    diagramElement41.noOfElements = 5;
    diagramElement41.isCheckedOutByMe = true;
    diagramElement41.isLocked = true;
    diagramElement41.isHasFormula = true;
    diagramElement41.isHasHyperlinks = true;
    diagramElement41.x = 255;
    diagramElement41.y = 150;
    diagramElement41.width = 68;
    diagramElement41.height = 48;
    diagramElement41.labelX = diagramElement41.x - 20;
    diagramElement41.labelY = diagramElement41.y + diagramElement41.height + 20;
    diagramElement41.labelWidth = 100;
    diagramElement41.labelHeight = 20;
    diagramElement41.borderColor = "Blue";
    diagramElement41.shape = ElementShape.Rectangle;

    var diagramElement42 = new DiagramElement();
    diagramElement42.imageSource = "./assets/images/Nodes_6.png";
    diagramElement42.name = "Node-7";
    diagramElement42.noOfElements = 5;
    diagramElement42.isCheckedOutByMe = true;
    diagramElement42.isLocked = true;
    diagramElement42.isHasFormula = true;
    diagramElement42.isHasHyperlinks = true;
    diagramElement42.x = 250;
    diagramElement42.y = 250;
    diagramElement42.width = 68;
    diagramElement42.height = 48;
    diagramElement42.labelX = diagramElement42.x - 20;
    diagramElement42.labelY = diagramElement42.y + diagramElement42.height + 20;
    diagramElement42.labelWidth = 100;
    diagramElement42.labelHeight = 20;
    //diagramElement42.borderColor = "Blue";
    diagramElement42.shape = ElementShape.Rectangle;

    var diagramElement5 = new DiagramElement();
    diagramElement5.imageSource = "./assets/images/Nodes_6.png";
    diagramElement5.name = "Node-4";
    diagramElement5.noOfElements = 35;
    diagramElement5.isHasKnowledge = false;
    diagramElement5.isCheckedOutByMe = false;
    diagramElement5.isLocked = true;
    diagramElement5.isHasFormula = true;
    diagramElement5.isHasHyperlinks = true;
    diagramElement5.x = 400;
    diagramElement5.y = 250;
    diagramElement5.width = 68;
    diagramElement5.height = 48;
    diagramElement5.labelX = diagramElement5.x - 20;
    diagramElement5.labelY = diagramElement5.y + diagramElement5.height + 20;
    diagramElement5.labelWidth = 100;
    diagramElement5.labelHeight = 20;
    diagramElement5.shape = ElementShape.Rectangle;

    var diagramElement52 = new DiagramElement();
    diagramElement52.imageSource = "./assets/images/Nodes_6.png";
    var text = "The diagram-view Angular component allows integrating the MindFusion.Diagramming API into Angular applications. It renders a Diagram instance assigned to diagram property as its model.";
    diagramElement52.name = "Node-52 - " + text;
    diagramElement52.noOfElements = 35;
    diagramElement52.isHasKnowledge = false;
    diagramElement52.isCheckedOutByMe = false;
    diagramElement52.isLocked = true;
    diagramElement52.isHasFormula = true;
    diagramElement52.isHasHyperlinks = true;
    diagramElement52.x = 400;
    diagramElement52.y = 350;
    diagramElement52.width = 68;
    diagramElement52.height = 48;
    diagramElement52.labelX = diagramElement52.x - 20;
    diagramElement52.labelY = diagramElement52.y + diagramElement52.height + 20;
    diagramElement52.labelWidth = 300;
    diagramElement52.labelHeight = 20;
    diagramElement52.shape = ElementShape.Rectangle;

    var diagramElement0 = new DiagramElement();
    diagramElement0.imageSource = "./assets/images/Nodes_6.png";
    diagramElement0.name = "Node-5";
    diagramElement0.noOfElements = 5;
    diagramElement0.isCheckedOutByMe = true;
    diagramElement0.isLocked = true;
    diagramElement0.isHasFormula = true;
    diagramElement0.isHasHyperlinks = true;
    diagramElement0.x = 500;
    diagramElement0.y = 150;
    diagramElement0.width = 68;
    diagramElement0.height = 48;
    diagramElement0.labelX = diagramElement0.x - 20;
    diagramElement0.labelY = diagramElement0.y + diagramElement0.height + 20;
    diagramElement0.labelWidth = 100;
    diagramElement0.labelHeight = 20;
    //diagramElement0.borderColor = "Blue";
    diagramElement0.shape = ElementShape.Rectangle;

    var diagramElement21 = new DiagramElement();
    diagramElement21.imageSource = "./assets/images/Nodes_6.png";
    diagramElement21.name = "Node-1 - My Name is longggggggggggggggggggggggggggggggggggg";
    diagramElement21.noOfElements = 5;
    diagramElement21.isCheckedOutByMe = true;
    diagramElement21.isLocked = true;
    diagramElement21.isHasFormula = true;
    diagramElement21.isHasHyperlinks = true;
    diagramElement21.x = 750;
    diagramElement21.y = 100;
    diagramElement21.width = 68;
    diagramElement21.height = 48;
    diagramElement21.labelX = diagramElement21.x - 20;
    diagramElement21.labelY = diagramElement21.y + diagramElement21.height + 20;
    diagramElement21.labelWidth = 360;
    diagramElement21.labelHeight = 20;
    diagramElement21.shape = ElementShape.Rectangle;

    var diagramElement22 = new DiagramElement();
    diagramElement22.imageSource = "./assets/images/Nodes_6.png";
    diagramElement22.name = "Node-2";
    diagramElement22.noOfElements = 5;
    diagramElement22.isCheckedOutByMe = true;
    diagramElement22.isLocked = true;
    diagramElement22.isHasFormula = true;
    diagramElement22.isHasHyperlinks = true;
    diagramElement22.x = 750;
    diagramElement22.y = 250;
    diagramElement22.width = 68;
    diagramElement22.height = 48;
    diagramElement22.labelX = diagramElement22.x - 20;
    diagramElement22.labelY = diagramElement22.y + diagramElement22.height + 20;
    diagramElement22.labelWidth = 100;
    diagramElement22.labelHeight = 20;
    diagramElement22.shape = ElementShape.Rectangle;

    var diagramElement23 = new DiagramElement();
    diagramElement23.imageSource = "./assets/images/Nodes_6.png";
    diagramElement23.name = "Node-3";
    diagramElement23.noOfElements = 5;
    diagramElement23.isCheckedOutByMe = true;
    diagramElement23.isLocked = true;
    diagramElement23.isHasFormula = true;
    diagramElement23.isHasHyperlinks = true;
    diagramElement23.x = 950;
    diagramElement23.y = 250;
    diagramElement23.width = 68;
    diagramElement23.height = 48;
    diagramElement23.labelX = diagramElement23.x - 20;
    diagramElement23.labelY = diagramElement23.y + diagramElement23.height + 20;
    diagramElement23.labelWidth = 100;
    diagramElement23.labelHeight = 20;
    diagramElement23.shape = ElementShape.Rectangle;

    var Shape = MindFusion.Diagramming.Shape;

    var diagramNodes: Array<CompositeNode> = [];

    var n01 = this.createCompositeNode(diagramElement01);
    var n02 = this.createCompositeNode(diagramElement02);
    var n0 = this.createCompositeNode(diagramElement0);
    var n11 = this.createCompositeNode(diagramElement11);
    //var n12 = this.createCompositeNode(diagramElement12);
    var n13 = this.createCompositeNode(diagramElement13);
    //var n14 = this.createCompositeNode(diagramElement14);
    var n15 = this.createCompositeNode(diagramElement15);
    var n12 = this.createCompositeNode(diagramElement31);
    var n3 = this.createCompositeNode(diagramElement32);
    var n32 = this.createCompositeNode(diagramElement33);
    var n61 = this.createCompositeNode(diagramElement61);
    var n62 = this.createCompositeNode(diagramElement62);
    var n63 = this.createCompositeNode(diagramElement63);
    var n41 = this.createCompositeNode(diagramElement41);
    var n42 = this.createCompositeNode(diagramElement42);
    var n5 = this.createCompositeNode(diagramElement5);
    var n52 = this.createCompositeNode(diagramElement52);
    var n21 = this.createCompositeNode(diagramElement21);
    var n22 = this.createCompositeNode(diagramElement22);
    var n23 = this.createCompositeNode(diagramElement23);

    diagramNodes.push(n01);
    diagramNodes.push(n02);
    diagramNodes.push(n0);
    diagramNodes.push(n11);
    diagramNodes.push(n13);
    diagramNodes.push(n15);
    diagramNodes.push(n12);
    diagramNodes.push(n3);
    diagramNodes.push(n32);
    diagramNodes.push(n61);
    diagramNodes.push(n62);
    diagramNodes.push(n63);
    diagramNodes.push(n41);
    diagramNodes.push(n42);
    diagramNodes.push(n5);
    diagramNodes.push(n21);
    diagramNodes.push(n22);
    diagramNodes.push(n23);

    this.diagramElements.push(diagramElement01);
    this.diagramElements.push(diagramElement02);
    this.diagramElements.push(diagramElement0);
    this.diagramElements.push(diagramElement11);
    this.diagramElements.push(diagramElement13);
    this.diagramElements.push(diagramElement15);
    this.diagramElements.push(diagramElement31);
    this.diagramElements.push(diagramElement32);
    this.diagramElements.push(diagramElement33);
    this.diagramElements.push(diagramElement61);
    this.diagramElements.push(diagramElement62);
    this.diagramElements.push(diagramElement63);
    this.diagramElements.push(diagramElement41);
    this.diagramElements.push(diagramElement42);
    this.diagramElements.push(diagramElement5);
    this.diagramElements.push(diagramElement21);
    this.diagramElements.push(diagramElement22);
    this.diagramElements.push(diagramElement23);

    var rectangleAnchorPoints = this.createAnchorPointsForRectangleShapeNode();
    var diamondAnchorPoints = this.createAnchorPointsForDiamondShapeNode();
    var circleAnchorPoints = this.createAnchorPointsForCircleShapeNode();

    for (var i = 0; i < diagramNodes.length; i++) {
      var diagramNode = diagramNodes[i];
      var diagramElement = diagramNode.getTag().DataObject as DiagramElement;
      if (diagramElement.shape == ElementShape.Rectangle) {
        diagramNode.setAnchorPattern(rectangleAnchorPoints);
      }
      else if (diagramElement.shape == ElementShape.Circle) {
        diagramNode.setAnchorPattern(circleAnchorPoints);
      }
      else if (diagramElement.shape == ElementShape.Decision) {
        diagramNode.setAnchorPattern(diamondAnchorPoints);
      }
    }

    this.backColorElement = diagramElement41;
    this.backColorNode = n41;

    this.createDiagramLink2(factory, n22, n23, 20, 12);

    //this.createDiagramLink2(factory, n41, n42, 6, 2);
    //this.createDiagramLink2(factory, n5, n0, 2, 18);
    //this.createDiagramLink2(factory, n21, n22, 6, 2);
    //this.createDiagramLink2(factory, n21, n23, 17, 15);

    var commentElement1 = new DiagramElement();
    commentElement1.name = "Mind Fusion Composite Nodes";
    commentElement1.x = 250;
    commentElement1.y = 10;
    commentElement1.width = 450;
    commentElement1.height = 50;
    commentElement1.isShadowElement = true;

    var commentElement2 = new DiagramElement();
    commentElement2.name = "Diagram Using Json Templates";
    commentElement2.x = 10;
    commentElement2.y = 420;
    commentElement2.width = 400;
    commentElement2.height = 50;
    commentElement2.shape = ElementShape.Ellipse;

    var commentElement3 = new DiagramElement();
    commentElement3.name = "Mind Fusion - Smart UI controls and libraries for web, desktop and mobile.";
    commentElement3.x = 5;
    commentElement3.y = 5;
    commentElement3.width = 100;
    commentElement3.height = 500;

    var commentElement4 = new DiagramElement();
    commentElement4.name = "aaaaaaaaaa aaaaaaaaa<aaaa aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa>aaaaaaaaaaaaaaaaaaaaa aaaaaaaaaaaaaaaaaaaa";
    commentElement4.x = 500;
    commentElement4.y = 450;
    commentElement4.width = 300;
    commentElement4.height = 80;

    this.createCommentNode(commentElement1);
    this.createCommentNode(commentElement2);
    this.createCommentNode(commentElement4);

    this.createTextNode();

    this.diagram.repaint();

    this.updateDiagramBounds();

    this.diagram.setUndoEnabled(false);
    this.diagram.setUndoEnabled(true);
  }

  public drawDiagram2(): void {

    this.isButtonVisible = false;

    this.diagramElements = new Array<DiagramElement>();

    this.clearDiagram();

    let factory = <Factory>this.diagram.getFactory();

    var diagramElement1 = new DiagramElement();
    diagramElement1.imageSource = "./assets/images/Nodes_6.png";
    diagramElement1.name = "Node-1";
    diagramElement1.noOfElements = 5;
    diagramElement1.isCheckedOutByMe = true;
    diagramElement1.isLocked = true;
    diagramElement1.isHasFormula = true;
    diagramElement1.isHasHyperlinks = true;
    diagramElement1.x = 100;
    diagramElement1.y = 250;
    diagramElement1.width = 68;
    diagramElement1.height = 48;
    diagramElement1.labelX = diagramElement1.x - 20;
    diagramElement1.labelY = diagramElement1.y + diagramElement1.height + 20;
    diagramElement1.labelWidth = 100;
    diagramElement1.labelHeight = 20;
    diagramElement1.shape = ElementShape.Rectangle;

    var diagramElement2 = new DiagramElement();
    diagramElement2.imageSource = "./assets/images/Nodes_6.png";
    diagramElement2.name = "Node-2";
    diagramElement2.noOfElements = 5;
    diagramElement2.isCheckedOutByMe = true;
    diagramElement2.isLocked = true;
    diagramElement2.isHasFormula = true;
    diagramElement2.isHasHyperlinks = true;
    diagramElement2.x = 300;
    diagramElement2.y = 350;
    diagramElement2.width = 68;
    diagramElement2.height = 48;
    diagramElement2.labelX = diagramElement2.x - 20;
    diagramElement2.labelY = diagramElement2.y + diagramElement2.height + 20;
    diagramElement2.labelWidth = 100;
    diagramElement2.labelHeight = 20;
    diagramElement2.shape = ElementShape.Rectangle;

    var Shape = MindFusion.Diagramming.Shape;

    var diagramNodes: Array<CompositeNode> = [];

    var node1 = this.createCompositeNode(diagramElement1);
    var node2 = this.createCompositeNode(diagramElement2);

    diagramNodes.push(node1);
    diagramNodes.push(node2);

    node1.setHandlesStyle(common.Diagramming.HandlesStyle.SquareHandles);
    node2.setHandlesStyle(common.Diagramming.HandlesStyle.SquareHandles);

    this.diagramElements.push(diagramElement1);
    this.diagramElements.push(diagramElement2);

    var rectangleAnchorPoints = this.createAnchorPointsForRectangleShapeNode();
    var diamondAnchorPoints = this.createAnchorPointsForDiamondShapeNode();
    var circleAnchorPoints = this.createAnchorPointsForCircleShapeNode();

    for (var i = 0; i < diagramNodes.length; i++) {
      var diagramNode = diagramNodes[i];
      var diagramElement = diagramNode.getTag().DataObject as DiagramElement;
      if (diagramElement.shape == ElementShape.Rectangle) {
        diagramNode.setAnchorPattern(rectangleAnchorPoints);
      }
      else if (diagramElement.shape == ElementShape.Circle) {
        diagramNode.setAnchorPattern(circleAnchorPoints);
      }
      else if (diagramElement.shape == ElementShape.Decision) {
        diagramNode.setAnchorPattern(diamondAnchorPoints);
      }
    }

    this.createDiagramLink2(factory, node1, node2, 20, 12);

    var commentElement1 = new DiagramElement();
    commentElement1.name = "Move the Node-2 below me";
    commentElement1.x = 500;
    commentElement1.y = 50;
    commentElement1.width = 400;
    commentElement1.height = 80;

    this.createCommentNode(commentElement1);

    this.diagram.repaint();

    this.updateDiagramBounds();
  }

  public createDiagramLink2(factory: Factory, sourceNode: CompositeNode, targetNode: CompositeNode, fromPortIndex: number, toPortIndex: number): void {

    if (sourceNode != null && targetNode != null) {

      var link = factory.createDiagramLink(sourceNode, targetNode);
      link.setStrokeThickness(0.75);
      link.setShadowColor("transparent");
      link.setHeadShapeSize(13);
      link.setHeadShape(Shape.fromId("PointerArrow"));
      link.setHeadBrush("Black");
      link.setWeight(10);
      link.setOriginAnchor(fromPortIndex);
      link.setDestinationAnchor(toPortIndex);
      link.setAutoRoute(true);

      //this.createDiagramLinkTag(link, sourceNode, targetNode, fromPortIndex, toPortIndex);

      link.route();

      this.consoleLinkPoints(link, "On Create:");
    }
  }

  public updateDiagramBounds(resizeToFit: boolean = false) {

    var xPositions: number[] = [];

    this.diagramElements.forEach(f => {
      xPositions.push(f.x!);
      xPositions.push(f.labelX!);
    });

    var yPositions: number[] = [];

    this.diagramElements.forEach(f => {
      yPositions.push(f.y!);
      yPositions.push(f.labelY!)
    });

    //if (!Utilities.isNullOrUndefined(this.diagramModel.comments)) {
    //  this.diagramModel.comments.forEach(f => {
    //    xPositions.push(f.xPosition);
    //    yPositions.push(f.yPosition);
    //  });
    //}
    var bounds = resizeToFit ? this.diagram.getContentBounds() : this.diagram.getBounds();

    var left: number = 0;
    var top: number = 0;

    if (xPositions.length > 0) {
      left = xPositions.reduce((a, b) => Math.min(a, b));
    }

    if (yPositions.length > 0) {
      top = yPositions.reduce((a, b) => Math.min(a, b));
    }

    if (left > 0) {
      left = 0;
    }

    if (top > 0) {
      top = 0;
    }

    var width = bounds.width;
    var height = bounds.height;

    var element = this.getParentDivElement()!;
    if (!Utilities.isNullOrUndefined(element)) {
      if (width < element.offsetWidth) {
        width = element.offsetWidth;
      }

      if (height < element.offsetHeight) {
        height = element.offsetHeight;
      }
    }

    if (resizeToFit) {
      if (width < (element.offsetWidth) * 100 / this.diagram.getZoomFactor())
        width = (element.offsetWidth) * 100 / this.diagram.getZoomFactor();
      if (height < (element.offsetHeight) * 98 / this.diagram.getZoomFactor())
        height = (element.offsetHeight) * 98 / this.diagram.getZoomFactor();
    }

    this.diagram.setBounds(new Rect(left, top, width, height));
  }

  public getParentDivElement() {

    var element = document.getElementById("diagramDiv");
    if (!Utilities.isNullOrUndefined(element)) {
      if (element.offsetHeight === 0 && element.offsetWidth === 0) {
        var elementClass = document.getElementsByClassName("diagramDiv");
        element = <HTMLElement>elementClass.item(elementClass.length - 1);
      }

      return element;
    }

    return undefined;
  }

  public createCompositeNode(diagramElement: DiagramElement): CompositeNode {

    var node = new this.DiagramNode();

    //node.setBounds(new Rect(diagramElement.x, diagramElement.y, diagramElement.width + 6, diagramElement.height + 6));
    node.setBounds(new Rect(diagramElement.x!, diagramElement.y!, diagramElement.width! + 0, diagramElement.height! + 0));
    node.setMainImage(diagramElement.imageSource);
    node.setLockImage("./assets/images/icons/Lock.png");
    node.setCheckoutImage("./assets/images/icons/Checkout.png");
    node.setKeyImage("./assets/images/icons/Key.png");
    node.setKnowledgeImage("./assets/images/icons/Knowledge.png");
    node.setProductImage("./assets/images/icons/ProductDefinition.png");
    node.setHyperlinkImage("./assets/images/icons/Hyperlink.png");

    var no: string = "";
    if (diagramElement.noOfElements) {
      no = diagramElement.noOfElements.toString();
    }
    node.setNoOfElementsText(no);
    node.setHandlesStyle(common.Diagramming.HandlesStyle.Custom);

    var compNode = node as CompositeNode;
    compNode.setClipToOutline(false);
    //compNode.setLocked(true);
    compNode.setId("1");
    var mainImageComponent = compNode.getComponent("MainImage") as any;
    mainImageComponent.image.width = diagramElement.width;
    mainImageComponent.image.height = diagramElement.height;

    if (diagramElement.comment)
      compNode.setTooltip(diagramElement.comment);
    else
      compNode.setTooltip(diagramElement.name!);

    this.setAdditionalIconsVisibility(compNode, diagramElement);

    var labelNode = this.createLabelNode(diagramElement, compNode);

    this.diagram.addItem(labelNode);
    node.attach(labelNode);

    var tag: any = { ElementType: ElementType.Element, DataObject: diagramElement, LabelNode: labelNode };
    node.setTag(tag);

    if (!diagramElement.borderColor) {
      diagramElement.borderColor = "Transparent";
    }

    var background = node.getComponent("OutlineShape");
    if (background) {

      this.setCompositeNodeShape(compNode, diagramElement);
    }

    this.diagram.addItem(node);

    return compNode;
  }

  public createTextNode() {

    var text = "The diagram-view Angular component allows integrating the MindFusion.Diagramming API into Angular applications. It renders a Diagram instance assigned to diagram property as its model.";
    var node = new ShapeNode(this.diagram);
    node.setBounds(new Rect(1100, 100, 170, 20));
    node.setText(text);
    node.setFont(new Font("Times New Roman", 15, false, false, false));
    node.setBrush("white");
    //node.setEnableStyledText(true);
    node.setTextAlignment(Alignment.Near);
    node.setShadowColor("transparent");
    node.resizeToFitText(common.Diagramming.FitSize.KeepWidth);
    node.setAllowIncomingLinks(false);
    node.setAllowOutgoingLinks(false);

    this.diagram.addItem(node);
  }

  public setAdditionalIconsVisibility(compNode: CompositeNode, diagramElement: DiagramElement): void {

    let width: number = diagramElement.width!;
    let height: number = diagramElement.height!;

    var knowledgeImageComponent = compNode.getComponent("KnowledgeImage") as any;
    knowledgeImageComponent.image.width = 8;
    knowledgeImageComponent.image.height = 7;

    var productImageComponent = compNode.getComponent("ProductImage") as any;
    var isProductImageVisible = diagramElement.isHasFormula;
    if (isProductImageVisible) {
      productImageComponent.image.width = 9;
      productImageComponent.image.height = 8;
    }
    else {
      productImageComponent.image.width = 0;
      productImageComponent.image.height = 0;
      productImageComponent.visibility = 0;
    }

    var top = height - 10;
    var noOfElementsTextComponent = compNode.getComponent("NoOfElementsText") as any;
    noOfElementsTextComponent.margin.top = top;
    var left = width - 18;

    var stackPanelComponent = compNode.getComponent("stackPanel") as any;
    stackPanelComponent.margin.left = left;
    stackPanelComponent.margin.top = 6;

    var isLockImageVisible = diagramElement.isLocked;
    var lockImageComponent = compNode.getComponent("LockImage") as any;

    var lockLeft = 0;
    var lockTop = 0;

    if (diagramElement.isTest == true) {
      lockLeft = -9;
      lockTop = 6;
    }

    if (isLockImageVisible) {
      lockImageComponent.image.width = 7;
      lockImageComponent.image.height = 7;
      lockImageComponent.margin.left = lockLeft;
      lockImageComponent.margin.top = lockTop;
    }
    else {
      lockImageComponent.image.width = 0;
      lockImageComponent.image.height = 0;
      lockImageComponent.visibility = 0;
    }

    var checkoutImageComponent = compNode.getComponent("CheckoutImage") as any;

    var checkoutLeft = 0;
    var checkoutTop = 0;

    if (diagramElement.isTest == true) {
      checkoutLeft = isLockImageVisible ? 6 : 0;
      checkoutTop = isLockImageVisible ? -6 : -6;
    }

    var isCheckoutImageVisible = diagramElement.isCheckedOutByMe;
    if (isCheckoutImageVisible) {
      checkoutImageComponent.image.width = 7;
      checkoutImageComponent.image.height = 7;
      checkoutImageComponent.margin.left = checkoutLeft;
      checkoutImageComponent.margin.top = checkoutTop;
    }
    else {
      checkoutImageComponent.image.width = 0;
      checkoutImageComponent.image.height = 0;
      checkoutImageComponent.visibility = 0;
    }

    var keyImageComponent = compNode.getComponent("KeyImage") as any;

    var keyLeft = 0;
    var keyTop = 2;

    if (diagramElement.isTest == true) {
      keyLeft = isLockImageVisible ? 0 : 0;
      keyTop = isLockImageVisible ? 1 : 7;
    }

    var isKeyImageVisible = !diagramElement.isCheckedOutByMe;
    if (isKeyImageVisible) {
      keyImageComponent.image.width = 8;
      keyImageComponent.image.height = 5;
      keyImageComponent.margin.left = keyLeft;
      keyImageComponent.margin.top = keyTop;
    }
    else {
      keyImageComponent.image.width = 0;
      keyImageComponent.image.height = 0;
      keyImageComponent.visibility = 0;
    }

    var hyperlinkLeft = width - 18;
    var hyperlinkTop = height;

    if (diagramElement.isTest == true) {
      hyperlinkLeft = hyperlinkLeft - 8;
    }
    var hyperlinkImageComponent = compNode.getComponent("HyperlinkImage") as any;
    hyperlinkImageComponent.image.width = 11;
    hyperlinkImageComponent.image.height = 9;
    hyperlinkImageComponent.margin.left = hyperlinkLeft;
    hyperlinkImageComponent.margin.top = hyperlinkTop;
  }

  public createLabelNode(diagramElement: DiagramElement, compNode: CompositeNode): ShapeNode {

    var labelY = compNode.getBounds().bottom() + 5;
    var node = this.diagram.getFactory().createShapeNode(diagramElement.labelX!, labelY, diagramElement.labelWidth!, diagramElement.labelHeight!);

    //node.setEnableStyledText(true);
    node.setText(diagramElement.name!);
    node.setFont(new Font("Verdana", 10));
    node.setAllowOutgoingLinks(false);
    node.setAllowIncomingLinks(false);
    node.setTransparent(true);
    node.setHandlesStyle(common.Diagramming.HandlesStyle.SquareHandles2);
    node.setObstacle(false);
    var style = node.getStyle();

    var tag: any = { ElementType: ElementType.Label, DataObject: diagramElement };
    node.setTag(tag);

    //if (diagramElement.comment)
    //  node.setTooltip(diagramElement.comment);
    //else
    //  node.setTooltip(diagramElement.name);

    return node;
  }

  public setCompositeNodeShape(compNode: CompositeNode, diagramElement: DiagramElement) {

    var shape = diagramElement.shape!.toString();

    (compNode as any).setOutlineShape(shape);

    //this.setCompositeNodeBorderColor(compNode, diagramElement.borderColor, false);
  }

  public createCommentNode(diagramElement: DiagramElement): ShapeNode {

    //var node = new ShapeNode(this.diagram);
    //node.setBounds(new Rect(diagramElement.x, diagramElement.y, diagramElement.width, diagramElement.height));
    var node = this.diagram.getFactory().createShapeNode(diagramElement.x!, diagramElement.y!, diagramElement.width!, diagramElement.height!);

    node.setText(diagramElement.name!);
    node.setFont(new Font("Verdana", 20, true, true, true));
    node.setHandlesStyle(MindFusion.Diagramming.HandlesStyle.SquareHandles2);
    node.setObstacle(false);
    node.setEnableStyledText(false);
    if (diagramElement.shape) {
      node.setShape(Shape.fromId(diagramElement.shape.toString()));
    }
    node.setShadowColor("Black");
    node.setStroke("LightGray");
    node.setStrokeThickness(1.5);
    node.setBrush("rgba(220, 220, 255, 0.6)");
    node.setShadowColor("Transparent");

    var tag: any = { ElementType: ElementType.Comment, DataObject: diagramElement };
    node.setTag(tag);

    this.diagram.addItem(node);

    if (diagramElement.isShadowElement) {
      node.setShadowColor("blue");
    }

    if (diagramElement.comment)
      node.setTooltip(diagramElement.comment);
    else
      node.setTooltip(diagramElement.name!);

    return node;
  }

  public createAnchorPointsForRectangleShapeNode(): AnchorPattern {

    var anchorSize: number = 6;
    var left = 0;
    var bottom = 98;
    var top = 0;
    var right = 102;
    var anchorPoints: AnchorPattern = new AnchorPattern([
      // Top . . .
      new AnchorPoint((102 / 6) * 1, top, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 2, top, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 3, top, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 4, top, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 5, top, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      // Bottom . . .
      new AnchorPoint((102 / 6) * 1, bottom, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 2, bottom, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 3, bottom, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 4, bottom, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint((102 / 6) * 5, bottom, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      // Left . . .
      new AnchorPoint(left, (100 / 6) * 1, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(left, (100 / 6) * 2, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(left, (100 / 6) * 3, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(left, (100 / 6) * 4, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(left, (100 / 6) * 5, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      // Right . . .
      new AnchorPoint(right, (100 / 6) * 1, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(right, (100 / 6) * 2, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(right, (100 / 6) * 3, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(right, (100 / 6) * 4, true, true, MarkStyle.Rectangle, "grey", anchorSize),
      new AnchorPoint(right, (100 / 6) * 5, true, true, MarkStyle.Rectangle, "grey", anchorSize),
    ]);
    return anchorPoints;
  }

  public createAnchorPointsForDiamondShapeNode(): AnchorPattern {

    var anchorSize: number = 6;

    var anchorPoints: AnchorPattern = new AnchorPattern([
      new AnchorPoint(0, (100 / 6) * 3, true, true, MarkStyle.Rectangle, "grey", anchorSize), // W
      new AnchorPoint((100 / 6) * 3, 0, true, true, MarkStyle.Rectangle, "grey", anchorSize), // N
      new AnchorPoint(100, (100 / 6) * 3, true, true, MarkStyle.Rectangle, "grey", anchorSize), // E
      new AnchorPoint((100 / 6) * 3, 100, true, true, MarkStyle.Rectangle, "grey", anchorSize), // S
    ]);
    return anchorPoints;
  }

  public createAnchorPointsForCircleShapeNode(): AnchorPattern {

    var anchorSize: number = 6;

    var anchorPoints: AnchorPattern = new AnchorPattern([
      new AnchorPoint(0, (110 / 6) * 3, true, true, MarkStyle.Rectangle, "grey", anchorSize), // W
      new AnchorPoint((114 / 6) * 1, 17, true, true, MarkStyle.Rectangle, "grey", anchorSize), // NW
      new AnchorPoint((100 / 6) * 3, 0, true, true, MarkStyle.Rectangle, "grey", anchorSize), // N
      new AnchorPoint((100 / 6) * 5, 17, true, true, MarkStyle.Rectangle, "grey", anchorSize), // NE
      new AnchorPoint(100, (110 / 6) * 3, true, true, MarkStyle.Rectangle, "grey", anchorSize), // E
      new AnchorPoint((100 / 6) * 5, 80, true, true, MarkStyle.Rectangle, "grey", anchorSize), // SE
      new AnchorPoint((100 / 6) * 3, 100, true, true, MarkStyle.Rectangle, "grey", anchorSize), // S
      new AnchorPoint((114 / 6) * 1, 86, true, true, MarkStyle.Rectangle, "grey", anchorSize), // SW
    ]);

    return anchorPoints;
  }

  public clearDiagram() {

    this.diagram.clearAll();

    (this.diagram as any).invalidate();
  }
}

class DiagramElement {
  constructor() {
    this.isCheckedOutByMe = false;
    this.isHasKnowledge = true;
    this.isTest = false;
  }

  id?: string;
  x?: number;
  y?: number;
  width?: number;
  height?: number;
  labelX?: number;
  labelY?: number;
  labelWidth?: number;
  labelHeight?: number;
  imageSource?: string;
  name?: string;
  comment?: string;
  shape?: ElementShape;
  noOfElements?: number;
  isHasKnowledge?: boolean;
  isCheckedOutByMe?: boolean;
  isLocked?: boolean;
  isHasFormula?: boolean;
  isHasHyperlinks?: boolean;
  borderColor?: string;
  isShadowElement?: boolean;
  isTest?: boolean;
}

class DiagramElementLink {

  sourceElement?: DiagramElement;
  targetElement?: DiagramElement;
  fromPortIndex?: number;
  toPortIndex?: number;
}

enum ElementType {

  Element = "Element",
  Label = "Label",
  Comment = "Comment",
  Link = "Link"
}
enum ElementShape {

  Rectangle = "Rectangle",
  Circle = "Circle",
  Decision = "Decision",
  Ellipse = "Ellipse"
}

class Utilities {

  static counter: number = 0;

  static getDiagramCounter(): number {

    return this.counter++;
  }

  static isNullOrUndefined<T>(obj: T | null | undefined): obj is null | undefined {
    return typeof obj === 'undefined' || obj === null;
  }

  static isNullOrEmptyString<T>(str: string): boolean {
    return str !== "";
  }

  static isEqual(source: string, comparer: string): boolean {
    return !this.isNullOrUndefined(source) && !this.isNullOrUndefined(comparer) && source.toString().toLowerCase() === comparer.toString().toLowerCase();
  }
}
