import { Injectable } from "@angular/core";
import * as MindFusion from "diagram-library";
import Diagram = MindFusion.Diagramming.Diagram;
//import * as Diagramming from '@mindfusion/diagramming';
//import * as Controls from '@mindfusion/controls';

@Injectable()
export class DiagramExportService {

  public diagram!: Diagram;
  public isBackgroundGridShown: boolean = true;

  public exportDiagram(withHighResolution: boolean = false, fileName: string = "canvas", type = "png"): void {

    var imgSrc = this.getCanvasImage(document, -1, type, withHighResolution, false);

    this.downloadImage(imgSrc, fileName, type);
  }

  public exportDiagramFit(reduceSize: number = 2, withHighResolution: boolean = false, fileName: string = "canvas", type = "png"): void {

    var imgSrc = this.getCanvasImage(document, reduceSize, type, withHighResolution, true);

    this.downloadImage(imgSrc, fileName, type);
  }

  private getCanvasImage(document: Document, reduceSize: number, type: string, withHighResolution: boolean, isFit: boolean) {

    if (this.isBackgroundGridShown) {
      this.hideBackgroundGrid(document);
    }

    var currentZoomFactor = this.diagram.getZoomFactor();

    this.diagram.setVirtualScroll(false);

    if (withHighResolution) {
      this.diagram.setZoomFactor(1000);
    }

    var canvas = document.querySelector('#diagram') as HTMLCanvasElement;

    if (isFit) {
      canvas = this.createFitCanvas(canvas, reduceSize);
    }

    var image = canvas.toDataURL("image/" + type, 1.0);

    if (withHighResolution) {
      this.diagram.setZoomFactor(currentZoomFactor);
    }

    this.diagram.setVirtualScroll(true);

    if (this.isBackgroundGridShown) {
      this.showBackgroundGrid(document);
    }

    return image;
  }

  private createFitCanvas(diagramCanvas: HTMLCanvasElement, reduceSize: number) {

    var exportCanvas = document.createElement("canvas");

    //var diagram2Canvas = <HTMLCanvasElement>document.getElementById("diagram2");
    //let diagramView2 = Diagramming.DiagramView.create(diagram2Canvas, this.diagram);

    return exportCanvas;
  }

  private createFitCanvas2(diagramCanvas: HTMLCanvasElement, reduceSize: number) {

    var sourceImageWidth = diagramCanvas.width;
    var sourceImageHeight = diagramCanvas.height;
    var destImageWidth = diagramCanvas.width / reduceSize;
    var destImageHeight = diagramCanvas.height / reduceSize;
    var destCanvasWidth = destImageWidth;
    var destCanvasHeight = destImageHeight;

    var exportCanvas = document.createElement("canvas");
    exportCanvas.width = destCanvasWidth;
    exportCanvas.height = destCanvasHeight;
    const ctx = exportCanvas.getContext("2d")!;
    ctx.drawImage(diagramCanvas,
      0, 0, sourceImageWidth, sourceImageHeight,
      0, 0, destImageWidth, destImageHeight);

    return exportCanvas;
  }

  private downloadImage(imgSrc: string, fileName: string, type: string) {
    const link = document.createElement('a');
    link.download = fileName + "." + type;
    link.href = imgSrc;
    link.click();
  }

  private hideBackgroundGrid(document: Document) {
    var diagramDiv = document.getElementById("gridImageDivElement");
    diagramDiv!.style.backgroundImage = "url()";
  }

  private showBackgroundGrid(document: Document) {
    var diagramDiv = document.getElementById("gridImageDivElement");
    diagramDiv!.style.backgroundImage = "url(./assets/images/GridImage.png)";
  }
}
